#!/usr/bin/perl

#
# Support.pm
# Copyright (C) 1991-2007 by John Heidemann <johnh@isi.edu>
# $Id: 01bcfe615d60b367cb9db88dc318fc22f9462f54 $
#
# This program is distributed under terms of the GNU general
# public license, version 2.  See the file COPYING
# in $dblibdir for details.
#

package Jdb::Support;

=head1 NAME

Jdb::Support - support routines for Jdb

=head1 SYNOPSIS

This class contains the bits of Jdb::Old that needed to be kept.

=head1 FUNCTIONS

=cut

@ISA = ();
($VERSION) = ('$Revision$' =~ m/(\d+)/);

## Module import.
use Exporter 'import';
@EXPORT = qw();
@EXPORT_OK = qw(
    code_prettify
    force_numeric
    progname
    $is_numeric_regexp
    );

#
# our libaries
#
use IO::Handle;
use IO::File;
use Carp qw(croak);

use Jdb::IO::Reader;
use Jdb::IO::Writer;

=head1 LOGGING REALTED FUNCTIONS

=head2 progname 

Generate the name of our program for error messages.

=cut
sub progname () {
    my($prog) = ($0);
    $prog =~ s@^.*/@@g;
    return $prog;
}

=head1 IO SETUP FUNCTIONS

=head2 default_in(@READER_OPTIONS)

Generate a default Jdb::Reader object with the given READER_OPTIONS

=cut
sub default_in ($@) {
    my $in_fh = new IO::Handle;
    $in_fh->fdopen(fileno(STDIN), "r") or croak progname . ": cannot open input as jdb.\n";
    my $in = new Jdb::IO::Reader(-fh => $in_fh, @_);
    return $in;
#    $in->error and croak progname . ": cannot open input as jdb.\n";
}

=head2 default_out(@WRITER_OPTIONS)

Generate a default Jdb::Writer object with the given READER_OPTIONS

=cut
sub default_out ($@) {
    my $out_fh = new IO::Handle;
    $out_fh->fdopen(fileno(STDOUT), "w+") or croak progname . ": cannot open stdout.\n";
    my $out = new Jdb::IO::Writer(-fh => $out_fh, @_);
    return $out;
#    $out->error and croak progname . ": cannot open STDOUT as jdb.\n";
}

=head1 CONVERSION FUNCTIONS

=head2 code_prettify 

Convert db-code into "pretty code".

=cut
sub code_prettify (@) {
    local($prettycode) = join(";", @_);
    $prettycode =~ s/\n/ /g;   # newlines will break commenting
    return $prettycode;
}

=head2 force_numeric

    my $x = force_numeric($s, $include_non_numeric)

Return $S if it's numeric, or undef if not.
If $INCLUDE_NON_NUMERIC, then non-numeric values register as zero.

=cut
# note that we tolerate spaces before and after,
# since field splitting doesn't always kill them
# (see TEST/dbcolstats_trailing_spaces.in)
our $is_numeric_regexp = '^\s*[+-]?(\d+\.\d+|\d+\.|\.\d+|\d+)([eE][+-]?\d+)?\s*$';
sub force_numeric {
    my($value, $zero_non_numeric) = @_;
    # next re is almost copied from L<perlretut>
    if ($value =~ /$is_numeric_regexp/) {
        return $value + 0.0;   # force numeric
    } else {
	if ($ignore_non_numeric) {
	    return undef;
	    next;
	} else {
	    return 0.0;
	};
    };
}


1;
